﻿using System;
using System.IO;
using MccDaq;

namespace AInScanForeground
{
    class Program
    {
        public const int NUMOFBLKS      = 8;
        public const int PACKETSIZE     = 32;
        public const int CHANCOUNT      = 1;
        public const int FIRSTCHANNEL   = 0;
        public const int LASTCHANNEL    = 0;
        public const int FREQ           = 1000;


   
        public const string DEVICE = "1208LS";
 
        public static StreamWriter fStream; 

        public static MccBoard daq;


        static void Main(string[] args)
        {

            int BoardNum = 0;
            int Rate = FREQ;
            Console.WriteLine("\nAnalog Input Foreground Scanning Example...\n");
            Console.WriteLine("\nLocating board...\n");

            BoardNum = GetBoardNum(DEVICE);

            if (BoardNum == -1)
            {
                Console.WriteLine("No {0} detected!", DEVICE);
                WaitForKey();
                return;
            }
  
            int BUFFERSIZE = PACKETSIZE * NUMOFBLKS * CHANCOUNT;
 
            daq = new MccDaq.MccBoard(BoardNum);
 
            IntPtr buffer = MccService.WinBufAllocEx(BUFFERSIZE);

            if (buffer == IntPtr.Zero)
            {
                Console.WriteLine("Bad Handle");
                WaitForKey();
                return;
            }

            //create text file to hold data
            fStream = new StreamWriter(@"C:\Users\Public\Documents\DataFile.asc");

            //writes Acquisition info to file header
            CreateFileHeaders();


            ScanOptions options = ScanOptions.ConvertData;

            //setup the acquisiton
            IsError(daq.AInScan(    FIRSTCHANNEL,
                                    LASTCHANNEL,
                                    BUFFERSIZE,
                                    ref Rate,
                                    Range.Bip10Volts,
                                    buffer,
                                    options)
                    );


            //declare ushort array to hold data returned by WinBufToArray
            ushort[] ushortArray = new ushort[BUFFERSIZE];

            //fetch data from driver
            IsError( MccService.WinBufToArray( buffer, ushortArray, 0, BUFFERSIZE ) );

            //Write data to console and to file
            DisplayData(ushortArray, BUFFERSIZE / CHANCOUNT);

            IsError(MccService.WinBufFreeEx(buffer));

            WaitForKey();
            
        }

        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static int GetBoardNum(string dev)
        {

            for (int BoardNum = 0; BoardNum < 99; BoardNum++)
            {
                MccDaq.MccBoard daq = new MccDaq.MccBoard(BoardNum);

                if (daq.BoardName.Contains(dev))
                {
                    Console.WriteLine("{0} board number = {1}\n", dev, BoardNum.ToString());
                    return BoardNum;
                }
            }
            return -1;
        }


        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static void WaitForKey()
        {
            Console.WriteLine("\nPress any key to continue...");
            do
            {
                System.Threading.Thread.Sleep(20);

            } while (!Console.KeyAvailable);
            ConsoleKeyInfo cki = Console.ReadKey();
        }

        /*////////////////////////////////////////////////////////////////////////////////////*/


        public static int IsError(ErrorInfo e)
        {
            if (e.Value != 0)
            {
                Console.WriteLine(e.Message);
                WaitForKey();
                return 1;
            }
            return 0;
        }


        /*////////////////////////////////////////////////////////////////////////////////////*/


        public static void DisplayData(ushort[] datArray, int rows)
        {
            
            int i = 0;
            float temp = 0;

            for (int row = 0; row < rows; row++)
            {
                Console.Write("{0}\t", row);
                for (int c = 0; c < CHANCOUNT; c++)
                {

                    IsError( daq.ToEngUnits(Range.Bip10Volts, datArray[i], out temp) );
                    Console.Write("{0}\t", temp.ToString("0.000").PadLeft(10));
                    fStream.Write("{0}\t", temp.ToString("0.000"));
                    i++;
                }
                Console.Write("\r\n");
                fStream.Write("\r\n");
            }
            fStream.Flush();
        }

        /*////////////////////////////////////////////////////////////////////////////////////*/

        public static void CreateFileHeaders()
        {
            Console.WriteLine("Reading channels {0} through {1}\n", FIRSTCHANNEL, LASTCHANNEL);

            //''''''''''''''''''''' create text file header strings ''''''''''''''''
            string[] hdr = new string[7];
            hdr[0] = "Recording date     : {0}";
            hdr[1] = "Block length       : {0}";
            hdr[2] = "Number of Packets  : {0}";
            hdr[3] = "Delta              : {0} sec.";
            hdr[4] = "Runtime            : {0} sec.";
            hdr[5] = "Number of channels : {0}";


            //''''''''''''''''''''''' build column headers string '''''''''''''''''''

            for (int j = FIRSTCHANNEL; j < LASTCHANNEL; j++)
            {
                hdr[6] = hdr[6] + String.Format("In{0} [V]", j); 
                hdr[6] = hdr[6] + "\t";

            }
            hdr[4].Trim('\t');

            //'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''
            DateTime MyDate = DateTime.Now;
            float tm =(float)((NUMOFBLKS * PACKETSIZE) / FREQ);
            fStream.WriteLine(hdr[0], MyDate);
            fStream.WriteLine(hdr[1], PACKETSIZE);
            fStream.WriteLine(hdr[2], NUMOFBLKS);
            fStream.WriteLine(hdr[3], (1.0f / FREQ).ToString("0.000"));
            fStream.WriteLine(hdr[4], tm.ToString("0.000")); 
            fStream.WriteLine(hdr[5], CHANCOUNT);
            fStream.WriteLine(hdr[6]);

            Console.WriteLine(hdr[0], MyDate);
            Console.WriteLine(hdr[1], PACKETSIZE);
            Console.WriteLine(hdr[2], NUMOFBLKS);
            Console.WriteLine(hdr[3], (1.0f / FREQ).ToString("0.000"));
            Console.WriteLine(hdr[4], tm.ToString("0.000"));  
            Console.WriteLine(hdr[5], CHANCOUNT);
            Console.WriteLine(hdr[6]);

        }

    }
}
